import { 
  Client, 
  Events, 
  GatewayIntentBits, 
  Message, 
  VoiceChannel, 
  ChannelType,
  Interaction,
  Guild,
  GuildMember,
  ButtonInteraction,
  VoiceState
} from 'discord.js';
import { config } from './config/config';
import { getRandomInt, randomChance } from './utils/random';
import { getRandomBehavior, getAudioFilePrefix } from './utils/bot-behaviors';
import { getRandomNickname, getTheme } from './utils/themes';
import { CatAudioPlayer } from './utils/audio-player';
import { AudioPlayerManager } from './utils/audio-player-manager';
import { guildSettings } from './utils/guild-settings';
import { deployCommands } from './utils/command-deployer';
import { commandsMap } from './commands';
import { StatusManager } from './utils/status-manager';
import fs from 'fs';
import path from 'path';
import { AUDIO_FILES_DIR } from './config/config';

// Create a new client instance with required intents
const client = new Client({
  intents: [
    GatewayIntentBits.Guilds,
    GatewayIntentBits.GuildMessages,
    GatewayIntentBits.MessageContent,
    GatewayIntentBits.GuildVoiceStates,
    GatewayIntentBits.GuildMembers,
  ]
});

// Initialize audio player manager
const audioPlayerManager = new AudioPlayerManager();

// Create status manager
const statusManager = new StatusManager(client, audioPlayerManager);

// Map to track voice channel join timers
const voiceJoinTimers = new Map<string, NodeJS.Timeout>();

// Deploy commands when starting
(async () => {
  try {
    await deployCommands();
  } catch (error) {
    console.error('Failed to deploy commands:', error);
  }
})();

// Handle new guild joins
client.on(Events.GuildCreate, async (guild: Guild) => {
  console.log(`Joined new guild: ${guild.name} (${guild.id})`);
  try {
    await guild.members.fetch();
    console.log(`Fetched ${guild.members.cache.size} members for new guild`);
    
    // Create default settings for this guild
    const settings = guildSettings.getSettings(guild.id);
    console.log(`Created settings for guild: ${guild.name} - Theme: ${settings.themeId}`);
    
    // Set a themed nickname
    const theme = getTheme(settings.themeId);
    const nickname = getRandomNickname(settings.themeId);
    try {
      const me = guild.members.cache.get(guild.client.user!.id);
      if (me && me.manageable) {
        await me.setNickname(`${nickname} ${theme.emojiIcon}`);
      }
    } catch (error) {
      console.error(`Failed to set nickname in new guild ${guild.id}:`, error);
    }
    
    // Update status to reflect new guild count
    statusManager.startStatusRotation();
    
    // Notify the server of the bot's presence
    const defaultChannel = guild.channels.cache.find(channel => 
      channel.type === ChannelType.GuildText && 
      (channel.name.includes('general') || channel.name.includes('bot') || channel.name.includes('welcome'))
    );
    
    if (defaultChannel?.isTextBased()) {
      defaultChannel.send(
        `${getRandomBehavior(guild.id)}\n\nHi everyone! I'm **${nickname}**, a bot that behaves like a ${theme.name.toLowerCase()}! ` +
        `${theme.emojiIcon} Use \`/theme\` to change what kind of animal I am, and \`/settings\` to configure my behavior!`
      );
    }
  } catch (error) {
    console.error(`Error setting up new guild:`, error);
  }
});

// Handle bot being removed from a guild
client.on(Events.GuildDelete, (guild: Guild) => {
  console.log(`Removed from guild: ${guild.name} (${guild.id})`);
  // Clear any timers for this guild
  if (voiceJoinTimers.has(guild.id)) {
    clearTimeout(voiceJoinTimers.get(guild.id));
    voiceJoinTimers.delete(guild.id);
  }
  
  // Update status to reflect new guild count
  statusManager.startStatusRotation();
});

// Handle new members joining
client.on(Events.GuildMemberAdd, (member: GuildMember) => {
  console.log(`New member joined: ${member.user.username} in ${member.guild.name}`);
});

// Handle voice state updates
client.on(Events.VoiceStateUpdate, (oldState: VoiceState, newState: VoiceState) => {
  // Only interested in the bot's voice state
  if (oldState.member?.user.id !== client.user?.id && newState.member?.user.id !== client.user?.id) {
    return;
  }
  
  // Check if the bot joined or left a voice channel
  const wasInVoice = !!oldState.channel;
  const isInVoice = !!newState.channel;
  
  // If voice state changed, update the status
  if (wasInVoice !== isInVoice) {
    console.log(`Bot ${isInVoice ? 'joined' : 'left'} a voice channel, updating status...`);
    statusManager.startStatusRotation(); // This will force an immediate update
  }
});

// When the client is ready, run this code (only once)
client.once(Events.ClientReady, async (readyClient: Client) => {
  console.log(`Ready! Logged in as ${readyClient.user?.tag}`);
  
  // Check if audio directory exists, create it if not
  if (!fs.existsSync(AUDIO_FILES_DIR)) {
    fs.mkdirSync(AUDIO_FILES_DIR, { recursive: true });
    console.log('Created audio directory. Please add cat sound files to it.');
  }
  
  // Start status rotation
  statusManager.startStatusRotation();
  
  // Fetch members for all guilds
  console.log(`Bot is in ${client.guilds.cache.size} guilds`);
  for (const guild of client.guilds.cache.values()) {
    try {
      console.log(`Fetching members for guild: ${guild.name} (${guild.id})`);
      await guild.members.fetch();
      console.log(`Successfully fetched ${guild.members.cache.size} members`);
      
      // Ensure guild has settings
      const settings = guildSettings.getSettings(guild.id);
      
      // Set the bot's nickname based on theme
      const theme = getTheme(settings.themeId);
      const nickname = getRandomNickname(settings.themeId);
      try {
        const me = guild.members.cache.get(client.user!.id);
        if (me && me.manageable) {
          await me.setNickname(`${nickname} ${theme.emojiIcon}`);
          console.log(`Set nickname to "${nickname} ${theme.emojiIcon}" in guild ${guild.name}`);
        }
      } catch (error) {
        console.error(`Failed to set nickname in guild ${guild.name}:`, error);
      }
    } catch (error) {
      console.error(`Error fetching members for guild ${guild.name}:`, error);
    }
  }
  
  // Start monitoring voice channels
  startVoiceChannelMonitoring();

  // List info about joined guilds
  console.log("Bot Information:");
  client.guilds.cache.forEach((guild: Guild) => {
    const settings = guildSettings.getSettings(guild.id);
    console.log(`Guild: ${guild.name} - Theme: ${settings.themeId} - Response: ${settings.responseChance}%`);
  });
});

// Handle slash commands
client.on(Events.InteractionCreate, async (interaction: Interaction) => {
  // Handle slash commands
  if (interaction.isCommand()) {
    const command = commandsMap.get(interaction.commandName);
    
    if (!command) return;
    
    try {
      await command.execute(interaction);
    } catch (error) {
      console.error(`Error executing command ${interaction.commandName}:`, error);
      
      if (interaction.replied || interaction.deferred) {
        await interaction.followUp({ content: 'There was an error executing this command!', ephemeral: true });
      } else {
        await interaction.reply({ content: 'There was an error executing this command!', ephemeral: true });
      }
    }
  }
  
  // Handle button interactions
  if (interaction.isButton()) {
    const buttonInteraction = interaction as ButtonInteraction;
    const customId = buttonInteraction.customId;
    
    // Handle theme change buttons
    if (customId.startsWith('theme_')) {
      const themeId = customId.split('_')[1];
      
      if (interaction.guildId) {
        if (guildSettings.setTheme(interaction.guildId, themeId)) {
          const theme = getTheme(themeId);
          const nickname = getRandomNickname(themeId);
          
          // Try to update the bot's nickname
          try {
            const guild = interaction.guild;
            if (guild) {
              const me = guild.members.cache.get(guild.client.user!.id);
              if (me && me.manageable) {
                await me.setNickname(`${nickname} ${theme.emojiIcon}`);
              }
            }
          } catch (error) {
            console.error(`Failed to update nickname in guild ${interaction.guildId}:`, error);
          }
          
          await buttonInteraction.update({
            content: `Theme updated to **${theme.name}**! I'm now **${nickname}** ${theme.emojiIcon}`,
            components: []
          });
        } else {
          await buttonInteraction.update({ content: 'Failed to update theme', components: [] });
        }
      }
    }
  }
});

// Handle messages
client.on(Events.MessageCreate, async (message: Message) => {
  // Ignore bot messages
  if (message.author.bot || !message.guildId) return;
  
  // Get response chance for this guild
  const responseChance = guildSettings.getSettings(message.guildId).responseChance;
  
  // Random chance to respond based on guild settings
  if (randomChance(responseChance)) {
    try {
      // Get a random behavior for this guild's theme
      const response = getRandomBehavior(message.guildId);
      
      // Send the response
      if (message.channel.isTextBased()) {
        await message.reply(response);
      }
    } catch (error) {
      console.error('Error responding to message:', error);
    }
  }
});

/**
 * Start monitoring voice channels for activity
 */
function startVoiceChannelMonitoring(): void {
  // Check voice channels regularly
  setInterval(() => {
    client.guilds.cache.forEach((guild: Guild) => {
      // Find voice channels with members
      const activeVoiceChannels = guild.channels.cache
        .filter((channel) => 
          channel.isVoiceBased() && 
          'members' in channel &&
          channel.members.size > 0 &&
          !channel.members.every((member: GuildMember) => member.user.bot)
        );
      
      if (activeVoiceChannels.size > 0) {
        // Randomly pick a voice channel to potentially join
        const channelID = guild.id;
        
        // If we don't have a timer for this guild, create one
        if (!voiceJoinTimers.has(channelID)) {
          const randomChannel = activeVoiceChannels.random();
          if (randomChannel && randomChannel.type === ChannelType.GuildVoice) {
            scheduleNextVoiceChannelJoin(guild.id, randomChannel as VoiceChannel);
          }
        }
      }
    });
  }, 10000); // Check every 10 seconds
}

/**
 * Schedule the next voice channel join for a guild
 */
function scheduleNextVoiceChannelJoin(guildId: string, initialChannel?: VoiceChannel): void {
  // Clear any existing timer
  if (voiceJoinTimers.has(guildId)) {
    const existingTimer = voiceJoinTimers.get(guildId);
    if (existingTimer) {
      clearTimeout(existingTimer);
    }
  }
  
  // Get random interval for next join
  const nextJoinDelay = getRandomInt(
    config.voiceChannelConfig.minJoinInterval,
    config.voiceChannelConfig.maxJoinInterval
  ) * 1000;
  
  // Schedule next join
  const timer = setTimeout(async () => {
    try {
      const guild = client.guilds.cache.get(guildId);
      
      if (guild) {
        // Find active voice channels
        const activeVoiceChannels = guild.channels.cache
          .filter((channel) => 
            channel.isVoiceBased() && 
            'members' in channel &&
            channel.members.size > 0 &&
            !channel.members.every((member: GuildMember) => member.user.bot)
          );
          
        if (activeVoiceChannels.size > 0) {
          // Choose a random active voice channel
          const randomChannel = activeVoiceChannels.random();
          if (randomChannel && randomChannel.type === ChannelType.GuildVoice) {
            const chosenChannel = randomChannel as VoiceChannel;
            
            // Get audio file prefix based on guild theme
            const audioFilePrefix = getAudioFilePrefix(guildId);
            
            // Join and play sounds
            await audioPlayerManager.joinChannel(
              chosenChannel,
              config.voiceChannelConfig.minStayDuration,
              config.voiceChannelConfig.maxStayDuration,
              config.voiceChannelConfig.minMeowInterval,
              config.voiceChannelConfig.maxMeowInterval,
              audioFilePrefix
            );
          }
        }
      }
    } catch (error) {
      console.error('Error joining voice channel:', error);
    }
    
    // Schedule next join
    scheduleNextVoiceChannelJoin(guildId);
  }, nextJoinDelay);
  
  // Store the timer
  voiceJoinTimers.set(guildId, timer);
  
  const guildName = client.guilds.cache.get(guildId)?.name || 'Unknown Guild';
  const channelName = initialChannel ? initialChannel.name : 'any active channel';
  console.log(`Scheduled voice channel join for guild ${guildName} (${guildId}) in ${nextJoinDelay / 1000} seconds`);
  if (initialChannel) {
    console.log(`Initial channel: ${initialChannel.name}`);
  } else {
    console.log(`No initial channel specified, will join any active channel`);
  }
}

// Log in to Discord with your token
client.login(config.token);

// Handle graceful shutdown
process.on('SIGINT', () => {
  console.log('Received SIGINT signal. Cleaning up...');
  audioPlayerManager.cleanup();
  process.exit(0);
});

process.on('SIGTERM', () => {
  console.log('Received SIGTERM signal. Cleaning up...');
  audioPlayerManager.cleanup();
  process.exit(0);
});
