import {
  AudioPlayer,
  AudioPlayerStatus,
  createAudioPlayer,
  createAudioResource,
  getVoiceConnection,
  joinVoiceChannel,
  VoiceConnectionStatus,
  NoSubscriberBehavior,
  VoiceConnection
} from '@discordjs/voice';
import { VoiceChannel } from 'discord.js';
import path from 'path';
import fs from 'fs';
import { AUDIO_FILES_DIR } from '../config/config';
import { getRandomElement, getRandomInt, sleep } from './random';
import { CAT_AUDIO_FILES } from './cat-behaviors';

export class CatAudioPlayer {
  private audioPlayer: AudioPlayer;
  private isPlaying: boolean = false;
  private shouldStop: boolean = false;
  private currentVoiceChannel: VoiceChannel | null = null;
  private currentConnection: VoiceConnection | null = null;

  constructor() {
    this.audioPlayer = createAudioPlayer({
      behaviors: {
        noSubscriber: NoSubscriberBehavior.Pause,
      }
    });
    
    this.audioPlayer.on(AudioPlayerStatus.Idle, () => {
      this.isPlaying = false;
    });
  }
  
  /**
   * Check if this player is currently in a voice channel
   */
  public isInVoiceChannel(): boolean {
    return this.currentVoiceChannel !== null;
  }
  
  /**
   * Clean up resources
   */
  public cleanup(): void {
    if (this.currentConnection) {
      try {
        this.currentConnection.destroy();
      } catch (error) {
        console.error('Error cleaning up voice connection:', error);
      }
      
      this.currentVoiceChannel = null;
      this.currentConnection = null;
    }
  }

  /**
   * Join a voice channel and start playing cat sounds
   */
  public async joinChannel(
    voiceChannel: VoiceChannel,
    minStayDuration: number,
    maxStayDuration: number,
    minMeowInterval: number,
    maxMeowInterval: number,
    audioFilePrefix: string = ''
  ): Promise<void> {
    try {
      // If already in a voice channel in this guild, leave it first
      const existingConnection = getVoiceConnection(voiceChannel.guild.id);
      if (existingConnection) {
        existingConnection.destroy();
      }
      
      // Store the current voice channel
      this.currentVoiceChannel = voiceChannel;
      
      // Create connection - using Discord.js's built-in adapter
      // The @ts-ignore is required because of type compatibility issues between Discord.js and @discordjs/voice
      // This is a common issue and is safe to ignore in this case
      const connection = joinVoiceChannel({
        channelId: voiceChannel.id,
        guildId: voiceChannel.guild.id,
        // @ts-ignore: Type compatibility issue between Discord.js and @discordjs/voice
        adapterCreator: voiceChannel.guild.voiceAdapterCreator,
      });
      
      // Store the connection
      this.currentConnection = connection;

      // Handle connection events
      connection.on(VoiceConnectionStatus.Disconnected, async () => {
        try {
          // Try to reconnect if disconnected unexpectedly
          await Promise.race([
            new Promise(resolve => connection.on(VoiceConnectionStatus.Ready, resolve)),
            new Promise((_, reject) => setTimeout(reject, 5000))
          ]);
        } catch (error) {
          // If we can't reconnect within 5 seconds, destroy the connection
          connection.destroy();
          this.currentVoiceChannel = null;
          this.currentConnection = null;
        }
      });

      // Subscribe the connection to the audio player
      connection.subscribe(this.audioPlayer);
      
      // Reset stop flag
      this.shouldStop = false;

      // Determine how long the cat will stay
      const stayDuration = getRandomInt(minStayDuration, maxStayDuration) * 1000;
      
      console.log(`Bot joined ${voiceChannel.name} in ${voiceChannel.guild.name} for ${stayDuration / 1000} seconds`);
      
      // Start playing sounds
      this.playRandomCatSounds(minMeowInterval, maxMeowInterval, stayDuration, audioFilePrefix);
      
      // Leave after duration
      setTimeout(() => {
        this.shouldStop = true;
        
        // Wait for any current sound to finish
        setTimeout(() => {
          if (this.currentConnection) {
            this.currentConnection.destroy();
            console.log(`Bot left ${voiceChannel.name} in ${voiceChannel.guild.name}`);
            this.currentVoiceChannel = null;
            this.currentConnection = null;
          }
        }, 1000);
      }, stayDuration);
      
    } catch (error) {
      console.error('Error joining voice channel:', error);
      this.currentVoiceChannel = null;
      this.currentConnection = null;
    }
  }

  /**
   * Play random cat sounds at random intervals
   */
  private async playRandomCatSounds(
    minInterval: number,
    maxInterval: number,
    maxDuration: number,
    audioFilePrefix: string = ''
  ): Promise<void> {
    let elapsedTime = 0;
    
    while (elapsedTime < maxDuration && !this.shouldStop) {
      // Play a random cat sound
      if (!this.isPlaying) {
        await this.playRandomSound(audioFilePrefix);
      }
      
      // Wait for a random interval
      const interval = getRandomInt(minInterval, maxInterval) * 1000;
      await sleep(interval);
      elapsedTime += interval;
    }
  }

  /**
   * Play a random cat sound file
   */
  private async playRandomSound(audioFilePrefix: string = ''): Promise<void> {
    try {
      // Filter audio files by prefix if provided
      let availableFiles = CAT_AUDIO_FILES;
      
      if (audioFilePrefix) {
        const prefixedFiles = CAT_AUDIO_FILES.filter(file => file.startsWith(audioFilePrefix));
        if (prefixedFiles.length > 0) {
          availableFiles = prefixedFiles;
        }
      }
      
      // Get a random audio file
      const audioFile = getRandomElement(availableFiles);
      const audioPath = path.join(AUDIO_FILES_DIR, audioFile);
      
      // Check if the file exists
      if (!fs.existsSync(audioPath)) {
        console.warn(`Audio file not found: ${audioPath}`);
        return;
      }
      
      // Create and play the audio resource
      const resource = createAudioResource(audioPath);
      this.isPlaying = true;
      this.audioPlayer.play(resource);
    } catch (error) {
      console.error('Error playing audio:', error);
      this.isPlaying = false;
    }
  }
}
